import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:angular_components/material_button/material_button.dart';
import 'package:angular_components/material_icon/material_icon.dart';
import 'package:angular_components/material_select/material_dropdown_select.dart';
import 'package:angular_components/model/selection/selection_model.dart';
import 'package:angular_components/model/ui/has_renderer.dart';
import 'package:redux_dev_tools/redux_dev_tools.dart';

import 'computed_state_component.dart';
import 'computed_state_selection_options.dart';

/// Provider the [DevToolsStore] to `<computed-state>` child components
const Provider<DevToolsStore> devToolsStoreProvider =
    const Provider<DevToolsStore>(DevToolsStore,
        useFactory: devToolsStoreFactory);

/// Factory for providing a [DevToolsStore].
DevToolsStore devToolsStoreFactory(ReduxDevToolsComponent devToolsComponent) =>
    devToolsComponent.store;

/// <redux-dev-tools>
///
/// Component for manipulating a [DevToolsStore] through a [MaterialDropdownSelectComponent].
/// required @Input(): [DevToolsStore store]
@Component(
  selector: 'redux-dev-tools',
  template: '''
    <material-dropdown-select
        #computedStateSelect
        [buttonText]="state?.currentAppState?.toString()"
        [selection]="selectionModel"
        [options]="selectionOptions"
        [componentRenderer]="componentRenderer">
      <div header>
        <material-button dense (trigger)="close()">Close</material-button>
        <material-button dense (trigger)="save()">Save</material-button>
        <material-button dense (trigger)="reset()">Reset</material-button>
        <material-button dense (trigger)="recompute()">Recompute</material-button>
      </div>
    </material-dropdown-select>
  ''',
  styles: const [
    'material-dropdown-select { font-family: "Lucida Console", Monaco, monospace; }'
  ],
  providers: const [
    materialProviders,
  ],
  directives: const [
    MaterialIconComponent,
    MaterialButtonComponent,
    MaterialDropdownSelectComponent,
  ],
)
class ReduxDevToolsComponent implements OnInit {
  /// Selection model used by [selectComponent].
  final SelectionModel<Object> selectionModel = new SelectionModel.withList();

  /// Selection options used by [selectComponent].
  ComputedStateSelectionOptions selectionOptions =
      new ComputedStateSelectionOptions([]);

  /// Dropdown menu displaying current and past states in a [DevToolsStore].
  @ViewChild('computedStateSelect')
  MaterialDropdownSelectComponent selectComponent;

  DevToolsStore _devToolsStore;

  /// Renderer used for displaying `computedStates` values/
  ComponentRenderer get componentRenderer => (value) => ComputedStateComponent;

  /// Returns the state from [_devToolsStore] or null.
  DevToolsState get state => _devToolsStore?.devToolsState;

  /// Getter for associated [DevToolsStore].
  DevToolsStore get store => _devToolsStore;

  @Input('store')
  set store(DevToolsStore store) {
    _devToolsStore = store;
    updateSelection();

    _devToolsStore.onChange.listen((_) => updateSelection());
    selectionModel.selectionChanges.listen((changesList) {
      if (!selectionModel.isSelected(state.currentAppState)) {
        store.dispatch(new DevToolsAction.jumpToState(
            state.computedStates.indexOf(selectionModel.selectedValues.first)));
      }
    });
  }

  /// Close the dropdown displayed by [selectComponent].
  void close() => selectComponent.close();

  @override
  void ngOnInit() => selectComponent.autoDismiss = false;

  /// Recompute states.
  void recompute() => _devToolsStore.dispatch(new DevToolsAction.recompute());

  /// Reset the store to the last initial or [save] state.
  void reset() => _devToolsStore.dispatch(new DevToolsAction.reset());

  /// Saves the current state as the initial state.
  void save() => _devToolsStore.dispatch(new DevToolsAction.save());

  /// Update selection when [_devToolsStore] is updated.
  void updateSelection() {
    selectionOptions = new ComputedStateSelectionOptions(state.computedStates);
    selectionModel.select(state.currentAppState);
  }
}
