@Tags(const ['aot'])
@TestOn('browser')
import 'dart:async';
import 'dart:html';

import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:angular_redux_dev_tools/redux_dev_tools_component.dart';
import 'package:angular_test/angular_test.dart';
import 'package:mockito/mockito.dart';
import 'package:pageloader/html.dart';
import 'package:pageloader/objects.dart';
import 'package:redux_dev_tools/redux_dev_tools.dart';
import 'package:test/test.dart';

/// Test bed for [ReduxDevToolsComponent].
NgTestBed<ReduxDevToolsComponent> devToolsBed;

/// Test fixture for [ReduxDevToolsComponent].
NgTestFixture<ReduxDevToolsComponent> devToolsFixture;

/// Page object for testing [ReduxDevToolsComponent]/
DevToolsPO devToolsPO;

/// [DevToolsStore] used in tests.
DevToolsStore devToolsStore;

final _intialState = 0;
final _mockDevToolsState = new MockDevToolsState();
final _mockDevToolsStore = new MockDevToolsStore();

/// Set up a real [DevToolsStore] with a simple counter app.
Future<Null> devToolsStoreSetUp() async {
  devToolsStore = new DevToolsStore(
      (state, action) => action is IncrementAction ? state + 1 : state,
      initialState: _intialState);
  await _baseSetUp();
}

/// Set up a [DevToolsStore] with a [Mock] object.
Future<Null> mockStoreSetUp() async {
  devToolsStore = _mockDevToolsStore;
  await _baseSetUp();
}

/// Set up mock responses used in testing.
void mockStoreSetUpAll() {
  when(_mockDevToolsState.computedStates).thenReturn([0, 1, 2, 3, 4]);
  when(_mockDevToolsState.currentAppState).thenReturn(2);
  when(_mockDevToolsStore.devToolsState).thenReturn(_mockDevToolsState);
  when(_mockDevToolsStore.onChange).thenReturn(const Stream.empty());
}

Future<Null> _baseSetUp() async {
  final parentDiv = new DivElement();
  final hostDiv = new DivElement();
  final overlayContainerDiv = new DivElement();
  parentDiv.children..add(hostDiv)..add(overlayContainerDiv);

  PageLoader Function(Element el, NgTestFixture fixture) pageLoader() =>
      (_, fixture) =>
          new HtmlPageLoader(parentDiv, executeSyncedFn: (fn) async {
            await fn();
            return fixture.update();
          });

  devToolsBed = new NgTestBed<ReduxDevToolsComponent>(host: hostDiv)
      .setPageLoader(pageLoader())
      .addProviders([
    materialProviders,
    provide(overlayContainerParent, useValue: overlayContainerDiv)
  ]);

  devToolsFixture = await devToolsBed.create(
      beforeChangeDetection: (c) => c.store = devToolsStore);
  devToolsPO = await devToolsFixture.resolvePageObject(DevToolsPO);
}

/// Page object for testing a `<redux-dev-tools>` component
class DevToolsPO {
  @ByCss('material-dropdown-select .button-text')
  PageLoaderElement _dropdown;

  @ByTagName('material-select-dropdown-item')
  List<PageLoaderElement> _dropdownItems;

  @ByTagName('material-button')
  List<PageLoaderElement> _headerButtons;

  /// Close button visible in open dropdown.
  FutureOr<bool> get closeButtonDisplayed => _closeButton?.displayed ?? false;

  /// Dropdown visible.
  FutureOr<bool> get dropdownDisplayed => _dropdown.displayed;

  /// String value of visible dropdown label-text.
  Future<String> get dropdownLabel => _dropdown.visibleText;

  /// Number of items in dropdown.
  int get dropdownLength => _dropdownItems.length;

  /// Recompute button visible in open dropdown.
  Future<bool> get recomputeButtonDisplayed =>
      _recomputeButton?.displayed ?? false;

  /// Reset button visible in open dropdown.
  Future<bool> get resetButtonDisplayed => _resetButton?.displayed ?? false;

  /// Save button visible in open dropdown.
  Future<bool> get saveButtonDisplayed => _saveButton?.displayed ?? false;

  PageLoaderElement get _closeButton =>
      _headerButtons.isNotEmpty ? _headerButtons[0] : null;

  PageLoaderElement get _recomputeButton =>
      _headerButtons.length > 3 ? _headerButtons[3] : null;

  PageLoaderElement get _resetButton =>
      _headerButtons.length > 2 ? _headerButtons[2] : null;

  PageLoaderElement get _saveButton =>
      _headerButtons.length > 1 ? _headerButtons[1] : null;

  /// Close button click in open dropdown.
  Future<Null> clickClose() => _closeButton.click();

  /// Dropdown click.
  Future<Null> clickDropdown() => _dropdown.click();

  /// Click on dropdown item by index [idx].
  Future<Null> clickDropdownItem(int idx) => _dropdownItems[idx].click();

  /// Click recompute in open dropdown.
  Future<Null> clickRecompute() => _recomputeButton.click();

  /// Click reset in open dropdown.
  Future<Null> clickReset() => _resetButton.click();

  /// Click save in open dropdown.
  Future<Null> clickSave() => _saveButton.click();
}

/// Type used for counter incrementing.
class IncrementAction {}

/// [Mock] object for [DevToolsState].
class MockDevToolsState extends Mock implements DevToolsState {}

/// [Mock] object for [DevToolsStore].
class MockDevToolsStore extends Mock implements DevToolsStore {}
